// Copyright (C) 2000 Open Source Telecom Corporation.
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "server.h"
#include <getopt.h>
#include <sys/wait.h>
#include <pwd.h>

static bool root = false;

#ifndef	HAVE_SETENV
static setenv(char *name, const char *value, int overwrite)
{
	char strbuf[256];

	sprintf(strbuf, "%s=%s", name, value);
	putenv(strbuf);
}
#endif

class KeyPolicies : public Keydata
{
public:
	KeyPolicies();
} keypolicies;

class KeyDriver : public Keydata
{
public:
	KeyDriver();
	const char *getDriverName(void);
} keydriver;

KeyServer keyserver;
KeyPaths keypaths;

KeyPolicies::KeyPolicies() :
Keydata("/bayonne/trunks")
{
};

KeyDriver::KeyDriver() :
Keydata("/bayonne/plugins")
{
};

KeyServer::KeyServer() :
Keydata("/bayonne/server")
{
	static KEYDEF defkeys[] = {
		{"user", "bayonne"},
		{"token", "&"},
		{"nodes", "1"},
		{NULL, NULL}};

	char *cp;
	char prefix[256];
	struct passwd *pwd;

	if(!getLast("node"))
	{
		gethostname(prefix, sizeof(prefix) - 1);
		cp = strchr(prefix, '.');
		if(cp)
			*cp = 0;
		setValue("node", prefix);
	}
	Load(defkeys);

	uid = getuid();
	gid = getgid();

	if(!geteuid())
		root = true;

	if(!geteuid() && getuid())	
	{
		root = true;
		pwd = getpwuid(uid);
		strcpy(prefix, getLast("wrapper"));
		cp = strtok(prefix, ",;:\t ");
		while(cp)
		{
			if(!strcmp(cp, pwd->pw_name))
				break;
			cp = strtok(NULL, ",;:\t ");
		}
		if(!cp)
		{
			slog(SLOG_ERROR) << "wrapper: suid: " << pwd->pw_dir << ": not permitted" << endl;
			exit(-1);
		}
	}

	pwd = getpwnam(getLast("user"));
	if(pwd && !geteuid())
	{
		gid = pwd->pw_gid;
		uid = pwd->pw_uid;
		setenv("HOME", pwd->pw_dir, 1);
	}

	endpwent();
	strcpy(prefix, getenv("HOME"));
	strcat(prefix, "/.bayonne");
	altdir = strdup(prefix);
}

KeyPaths::KeyPaths() :
Keydata("/bayonne/paths")
{ 
        static KEYDEF defpaths[] = {
        {"libpath", "/usr/lib/bayonne"},
        {"libexec", "/usr/lib"},
        {"tgipath", "/usr/lib/tgi:/bin:/usr/bin"},
        {"datafiles", "/var/bayonne"},
        {"scripts", "/usr/share/aascripts"},
        {"prompts", "/usr/share/aaprompts"},
        {"spool", "/var/spool/bayonne"},
        {"runfiles", "/var/run"},
        {"precache", "/var/bayonne/cache"},
        {"config", "bayonne runtime configuration"},
        {NULL, NULL}};                                                         

	Load(defpaths);
 
}                                                    

void KeyServer::setUid(void)
{
	setuid(uid);
	uid = getuid();
}

void KeyServer::setGid(void)
{
	setgid(gid);
	gid = getgid();
}

int main(int argc, char **argv)
{
	char buffer[256];
	char *cp;
	int fd;

	if(getenv("CONFIG_KEYDATA"))
	{
		slog(SLOG_ERROR) << "wrapper: config redirect not permitted" << endl;
		exit(-1);
	}

	keyserver.setGid();
	keyserver.setUid();

	chdir(keypaths.getDatafiles());
	slog.open("bayonne", SLOG_DAEMON);


	sprintf(buffer, "%s/tgi", keypaths.getLibexec());
	setenv("SERVER_LIBEXEC", buffer, 1);
	setenv("SERVER_SOFTWARE", "bayonne", 1);
	setenv("SERVER_POLICIES", keypolicies.getLast("groups"), 1);
	setenv("SERVER_PROTOCOL", "2.2", 1);
	setenv("SERVER_VERSION", VERPATH, 1);
	setenv("SERVER_PLATFORM", keydriver.getLast("driver"), 1);
	setenv("SERVER_TOKEN", keyserver.getLast("token"), 1);
	setenv("SERVER_NODE", keyserver.getLast("node"), 1);

	strcpy(buffer, keyserver.getPrefix());
	strcat(buffer, "/tgi");
	if(isDir(buffer))
	{
		strcat(buffer, ":");
		strcat(buffer, keypaths.getTgipath());
		setenv("PATH", buffer, 1);
	}
	else
		setenv("PATH", keypaths.getTgipath(), 1);

	sprintf(buffer, "%s/.bayonne", getenv("HOME"));
	setenv("SERVER_CONTROL", buffer, 1);

	strcpy(buffer, keypaths.getRunfiles());
	strcat(buffer, "/bayonne.ctrl");
	if(canModify(buffer))
	{
		strcpy(buffer, keypaths.getRunfiles());
		strcat(buffer, "/bayonne");
		setenv("SERVER_CONTROL", buffer, 1);
	}

	if(argc < 2)
	{
		cerr << "use: bayonne_wrapper exec..." << endl;
		exit(-1);
	}
	++argv;

	if(**argv == '-')
		++*argv;
	else
	{
		sprintf(buffer, "%s.ctrl", getenv("SERVER_CONTROL"));
		fd = open(buffer, O_WRONLY);
		if(fd < 0)
		{
			slog(SLOG_ERROR) << "wrapper: " << buffer << ": cannot open fifo" << endl;
			exit(-1);
		}
		dup2(fd, 1);
		close(fd);
	}

	if(root)
	{
		sprintf(buffer, "%s/%s", getenv("SERVER_LIBEXEC"), *argv);
		*argv = buffer;
		execv(*argv, argv);
	} 
	else
		execvp(*argv, argv);

	slog(SLOG_ERROR) << "wrapper: " << *argv << ": cannot execute" << endl;
	exit(-1);
}

